package key_test

import (
	"strings"
	"testing"

	"git.sr.ht/~minus/sshign/key"
)

func TestUnencryptedHappy(t *testing.T) {
	t.Parallel()

	priv := privRSAKey(t, 2048)
	msg := []byte("I am a message!")
	s, err := key.Sign(nil, newNamedBuffer("/foo", privRSAPEM(t, priv, nil)), newNamedBuffer("stdin", msg))
	if err != nil {
		t.Fatalf("Expected signature error to be nil; got %v", err)
	}
	err = key.Verify(
		newNamedBuffer("/foo.pub", pubRSAKey(t, priv)),
		newNamedBuffer("/bar", []byte(s)),
		newNamedBuffer("stdin", msg),
	)
	if err != nil {
		t.Fatalf("Expected verify error to be nil; got %v", err)
	}
}

func TestUnencryptedFailedVerify(t *testing.T) {
	t.Parallel()

	priv := privRSAKey(t, 2048)
	msg := []byte("I am a message!")
	s, err := key.Sign(nil, newNamedBuffer("/id_rsa", privRSAPEM(t, priv, nil)), newNamedBuffer("stdin", msg))
	if err != nil {
		t.Fatalf("Expected signature error to be nil; got %v", err)
	}
	err = key.Verify(
		newNamedBuffer("/id_rsa.pub", pubRSAKey(t, privRSAKey(t, 2048))),
		newNamedBuffer("/some/sign/file", []byte(s)),
		newNamedBuffer("stdin", msg),
	)
	if err == nil {
		t.Fatalf("Expected verify error to be non-nil")
	}
	if _, ok := err.(key.VerifyError); !ok {
		t.Fatalf("Expected verify error to be of type verifyError; got %T", err)
	}
}

func TestEncryptedHappy(t *testing.T) {
	t.Parallel()

	priv := privRSAKey(t, 2048)
	msg := []byte("I am a message!")
	pass := []byte("this is a great password")
	req := newMockRequester()
	req.PassphraseOutput.Ret0 <- pass
	req.PassphraseOutput.Ret1 <- nil
	s, err := key.Sign(req, newNamedBuffer("/foo", privRSAPEM(t, priv, pass)), newNamedBuffer("stdin", msg))
	if err != nil {
		t.Fatalf("Expected signature error to be nil; got %v", err)
	}
	select {
	case <-req.PassphraseCalled:
		path := <-req.PassphraseInput.Path
		if path != "/foo" {
			t.Fatalf("Expected passphrase to be requested for file /foo; got %s", path)
		}
	default:
		t.Fatalf("Expected passphrase to be requested")
	}
	err = key.Verify(
		newNamedBuffer("/foo.pub", pubRSAKey(t, priv)),
		newNamedBuffer("/bar", []byte(s)),
		newNamedBuffer("stdin", msg),
	)
	if err != nil {
		t.Fatalf("Expected verify error to be nil; got %v", err)
	}
}

func TestEncryptedBassPass(t *testing.T) {
	t.Parallel()

	priv := privRSAKey(t, 2048)
	msg := []byte("I am a message!")
	pass := []byte("this is a great password")
	req := newMockRequester()
	req.PassphraseOutput.Ret0 <- []byte("foo")
	req.PassphraseOutput.Ret1 <- nil
	_, err := key.Sign(req, newNamedBuffer("/foo", privRSAPEM(t, priv, pass)), newNamedBuffer("stdin", msg))
	if err == nil {
		t.Fatalf("Expected a bad password to fail key.Sign")
	}
	if !strings.Contains(err.Error(), "decryption password incorrect") {
		t.Fatalf("Expected a bad password to cause an 'Invalid Passphrase' error; got %s", err)
	}

	select {
	case <-req.PassphraseCalled:
		path := <-req.PassphraseInput.Path
		if path != "/foo" {
			t.Fatalf("Expected passphrase to be requested for file /foo; got %s", path)
		}
	default:
		t.Fatalf("Expected passphrase to be requested")
	}
}
