package key

import (
	"crypto/rand"
	"encoding/base64"
	"fmt"
	"io/ioutil"

	"golang.org/x/crypto/ssh"
)

// Requester is a type which can request information from a user
// dynamically.
type Requester interface {
	// Passphrase requests a passphrase for a file at path.
	Passphrase(path string) ([]byte, error)
}

func parsePrivkey(r Requester, f File) (ssh.Signer, error) {
	skData, err := ioutil.ReadAll(f)
	if err != nil {
		return nil, fmt.Errorf("could not read secret key %s: %w", f.Name(), err)
	}

	sk, err := ssh.ParsePrivateKey(skData)
	if _, ok := err.(*ssh.PassphraseMissingError); !ok {
		return sk, err
	}

	pass, err := r.Passphrase(f.Name())
	if err != nil {
		return nil, fmt.Errorf("unable to request passphrase for %s: %w", f.Name(), err)
	}

	sk, err = ssh.ParsePrivateKeyWithPassphrase(skData, pass)
	if err != nil {
		return nil, fmt.Errorf("unable to parse encrypted key %s: %w", f.Name(), err)
	}
	return sk, nil
}

// Sign is used to sign a file using a private key.  A password will be requested
// if privKey is encrypted.
func Sign(req Requester, privKey, toSign File) (string, error) {
	sk, err := parsePrivkey(req, privKey)
	if err != nil {
		return "", fmt.Errorf("could not parse secret key %s: %w", privKey.Name(), err)
	}

	msg, err := ioutil.ReadAll(toSign)
	if err != nil {
		return "", fmt.Errorf("could not read message file %s: %w", privKey.Name(), err)
	}

	sig, err := sk.Sign(rand.Reader, msg)
	if err != nil {
		return "", fmt.Errorf("could not sign message: %w", err)
	}

	return fmt.Sprintf("%s %s", sig.Format, base64.StdEncoding.EncodeToString(sig.Blob)), nil
}
