package key

import (
	"bytes"
	"encoding/base64"
	"fmt"
	"io/ioutil"
	"strings"

	"golang.org/x/crypto/ssh"
)

// VerifyError is an error type signaling that verification failed.
type VerifyError struct {
	child error
}

// Error implements error.
func (e VerifyError) Error() string {
	return fmt.Sprintf("failed: %s", e.child)
}

// Verify verifies a message using a signature file and a public
// ssh key.
func Verify(pubKey, signature, msg File) error {
	pkData, err := ioutil.ReadAll(pubKey)
	if err != nil {
		return fmt.Errorf("could not read public key %s: %w", pubKey.Name(), err)
	}
	d := bytes.SplitN(pkData, []byte(" "), 3)
	if len(d) < 2 {
		return fmt.Errorf("could not parse public key: expected format to at least match '<algorithm> <pubkey>'")
	}
	pkData = d[1]
	n, err := base64.StdEncoding.Decode(pkData, pkData)
	if err != nil {
		return fmt.Errorf("could not base64-decode public key %s: %w", pubKey.Name(), err)
	}
	pkData = pkData[:n]
	pk, err := ssh.ParsePublicKey(pkData)
	if err != nil {
		return fmt.Errorf("could not parse public key %s: %w", pubKey.Name(), err)
	}

	sigData, err := ioutil.ReadAll(signature)
	if err != nil {
		return fmt.Errorf("could not read signature file %s: %w", signature.Name(), err)
	}
	sigParts := strings.SplitN(string(sigData), " ", 2)
	if len(sigParts) < 2 {
		return fmt.Errorf("could not parse signature file: expected format '<algorithm> <signature>'")
	}
	sigData, err = base64.StdEncoding.DecodeString(sigParts[1])
	if err != nil {
		return fmt.Errorf("could not base64-decode signature file %s: %w", signature.Name(), err)
	}
	sig := ssh.Signature{
		Format: sigParts[0],
		Blob:   sigData,
	}

	msgData, err := ioutil.ReadAll(msg)
	if err != nil {
		return fmt.Errorf("could not read message file %s: %w", msg.Name(), err)
	}

	if err := pk.Verify(msgData, &sig); err != nil {
		return VerifyError{child: err}
	}
	return nil
}
